@@ -2,6 +2,8 @@
 This **module** is used to load and read
 """
 import os
+import warnings
+import re
 
 import numpy as np
 
@@ -81,14 +83,102 @@ def _mol2_bond(line, current_molecule, atom_residue_map):
                     atom_residue_map[words[2]][1].type.tail = atom_residue_map[words[2]][0]
 
 
-def load_mol2(filename, ignore_atom_type=False):
+def _mol2_template_atom(line, current_residue_index, temp, current_residue, atom_residue_map):
+    """
+
+    :param line:
+    :param current_residue_index:
+    :param current_residue:
+    :param ignore_atom_type:
+    :param temp:
+    :param current_molecule:
+    :param atom_residue_map:
+    :return:
+    """
+    words = line.split()
+    if current_residue_index is None or int(words[6]) != current_residue_index:
+        current_residue_index = int(words[6])
+        if words[7] not in ResidueType.get_all_types():
+            set_real_global_variable(words[7], ResidueType(name=words[7]))
+            temp = True
+        else:
+            temp = False
+        current_residue = ResidueType.get_type(words[7])
+
+    temp_atom_type = AtomType.get_type(words[5])
+
+    if temp:
+        current_residue.Add_Atom(words[1], temp_atom_type, *words[2:5])
+        current_residue.atoms[-1].Update(**{"charge[e]": float(words[8])})
+    atom_residue_map[words[0]] = [words[1], current_residue, current_residue_index, temp]
+    return current_residue_index, current_residue, temp
+
+
+def _mol2_template_bond(line, atom_residue_map):
+    """
+
+    :param line:
+    :param current_molecule:
+    :param atom_residue_map:
+    :return:
+    """
+    words = line.split()
+    atom1_index, atom2_index = words[1], words[2]
+    atom_name, residue_type, residue_index, is_new = 0, 1, 2, 3
+    atom1_info, atom2_info = atom_residue_map[atom1_index], atom_residue_map[atom2_index]
+    if atom1_info[residue_type] == atom2_info[residue_type] and atom1_info[is_new]:
+        atom1_info[residue_type].Add_Connectivity(atom1_info[atom_name], atom2_info[atom_name])
+    else:
+        index_diff = atom1_info[residue_index] - atom2_info[residue_index]
+        if abs(index_diff) == 1:
+            if atom1_info[is_new]:
+                if index_diff < 0:
+                    atom1_info[residue_type].tail = atom1_info[atom_name]
+                else:
+                    atom1_info[residue_type].head = atom1_info[atom_name]
+            if atom2_info[is_new]:
+                if index_diff < 0:
+                    atom2_info[residue_type].head = atom2_info[atom_name]
+                else:
+                    atom2_info[residue_type].tail = atom2_info[atom_name]
+
+
+def _mol2_template(filename):
+    """
+    This is used to make loading more efficient when importing the force fields
+
+    :param filename:
+    :return:
+    """
+    current_residue_index = None
+    current_residue = None
+    atom_residue_map = Xdict()
+    temp = None
+    with open(filename) as f:
+        flag = None
+        for line in f:
+            if not line.strip():
+                continue
+            if line.startswith("@<TRIPOS>"):
+                flag = line[9:].strip()
+            elif flag == "ATOM":
+                current_residue_index, current_residue, temp = _mol2_template_atom(line, current_residue_index, temp,
+                                                                                   current_residue, atom_residue_map)
+            elif flag == "BOND":
+                _mol2_template_bond(line, atom_residue_map)
+
+def load_mol2(filename, ignore_atom_type=False, as_template=False):
     """
     This **function** is used to load a mol2 file
 
     :param filename: the name of the file to load
     :param ignore_atom_type: ignore the atom types in the mol2 file
-    :return: a Molecule instance
+    :param as_template: only read the mol2 file as some residue types and no molecule will created
+        **New From 1.2.6.8**
+    :return: a Molecule instance if as_template is False
     """
+    if as_template:
+        return _mol2_template(filename)
     with open(filename) as f:
         # 存储读的时候的临时信息，key是编号
         # value是list：原子名(0)、residue(1)、residue编号(2)、是否是新的residue type(3)、该原子(4)、residue type的最新原子(5)
@@ -149,13 +239,57 @@ def _pdb_ssbond_after(chain, ssbonds, molecule):
         res_a_index = chain[ssbond[15]][int(ssbond[17:21])]
         res_b_index = chain[ssbond[29]][int(ssbond[31:35])]
         if res_a_index > res_b_index:
-            res_a_index, res_b_index = (res_b_index, res_a_index)
+            res_a_index, res_b_index = res_b_index, res_a_index
         res_a = molecule.residues[res_a_index]
         res_b = molecule.residues[res_b_index]
         molecule.Add_Residue_Link(res_a.name2atom(res_a.type.connect_atoms["ssbond"]),
                                   res_b.name2atom(res_b.type.connect_atoms["ssbond"]))
 
 
+def _pdb_link_after(chain, links, molecule):
+    """
+
+    :param chain:
+    :param links:
+    :param molecule:
+    :return:
+    """
+    for link in links:
+        chain_a, chain_b = link[21], link[51]
+        res_a_pdb, res_b_pdb = int(link[22:26]), int(link[52:56])
+        if chain_a not in chain or chain_b not in chain or \
+                res_a_pdb not in chain[chain_a] or res_b_pdb not in chain[chain_a]:
+            warnings.warn(f"The link between {chain_a}{res_a_pdb} and {chain_b}{res_b_pdb} is not valid")
+            continue
+        res_a_index = chain[link[21]][int(link[22:26])]
+        res_b_index = chain[link[51]][int(link[52:56])]
+        if res_a_index > res_b_index:
+            res_a_index, res_b_index = res_b_index, res_a_index
+        res_a = molecule.residues[res_a_index]
+        res_b = molecule.residues[res_b_index]
+        molecule.Add_Residue_Link(res_a.name2atom(link[12:16].strip()),
+                                  res_b.name2atom(link[42:46].strip()))
+
+
+def _pdb_add_atom(current_residue, atomname, x, y, z,
+                  ignore_hydrogen, ignore_unknown_name, atom_map, atom_index):
+    """
+    add the atom to the residue
+    """
+    if ignore_hydrogen and (atomname.startswith("H") or
+                            (len(atomname) > 1 and atomname[0] in "123" and atomname[1] == "H")):
+        return False
+    try:
+        current_residue.Add_Atom(atomname, x=x, y=y, z=z)
+        if atom_index not in atom_map:
+            atom_map[atom_index] = current_residue.atoms[-1]
+        return True
+    except KeyError as ke:
+        if ignore_unknown_name:
+            return False
+        raise ke
+
+
 def _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore_hydrogen, ignore_unknown_name):
     """
 
@@ -167,8 +301,10 @@ def _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore
     :param ignore_unknown_name:
     :return:
     """
+    atom_map = Xdict()
     current_residue_count = -1
     current_residue_index = None
+    current_insertion_code = None
     current_residue = None
     current_resname = None
     links = []
@@ -176,14 +312,16 @@ def _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore
         for line in f:
             if line.startswith("ATOM") or line.startswith("HETATM"):
                 extra = line[16]
+                insertion_code = line[26]
                 resindex = int(line[22:26])
                 resname = line[17:20].strip()
                 atomname = line[12:16].strip()
+                atom_index = int(line[6:11])
                 x = float(line[30:38])
                 y = float(line[38:46])
                 z = float(line[46:54])
                 if current_residue_index is None or current_residue_index != resindex or \
-                        current_resname != resname:
+                        current_resname != resname or current_insertion_code != insertion_code:
                     current_residue_count += 1
                     if current_residue:
                         molecule.Add_Residue(current_residue)
@@ -192,17 +330,12 @@ def _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore
                             links.append(len(molecule.residues))
                     current_residue = Residue(ResidueType.get_type(residue_type_map[current_residue_count]))
                     current_residue_index = resindex
+                    current_insertion_code = insertion_code
                     current_resname = resname
                 if extra not in (" ", position_need):
                     continue
-                if ignore_hydrogen and atomname.startswith("H"):
-                    continue
-                try:
-                    current_residue.Add_Atom(atomname, x=x, y=y, z=z)
-                except KeyError as ke:
-                    if ignore_unknown_name:
-                        continue
-                    raise ke
+                _pdb_add_atom(current_residue, atomname, x, y, z,
+                              ignore_hydrogen, ignore_unknown_name, atom_map, atom_index)
             elif line.startswith("TER"):
                 current_residue_index = None
                 current_resname = None
@@ -213,6 +346,7 @@ def _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore
         res_a = molecule.residues[count]
         res_b = molecule.residues[count - 1]
         molecule.Add_Residue_Link(res_a.name2atom(res_a.type.head), res_b.name2atom(res_b.type.tail))
+    return atom_map
 
 
 def _pdb_judge_histone(judge_histone, residue_type_map, current_histone_information):
@@ -234,58 +368,157 @@ def _pdb_judge_histone(judge_histone, residue_type_map, current_histone_informat
         current_histone_information = {"DeltaH": False, "EpsilonH": False}
 
 
-def load_pdb(filename, judge_histone=True, position_need="A", ignore_hydrogen=False, ignore_unknown_name=False):
+def _pdb_read_sequences(line, sequences):
+    """
+
+    :param line:
+    :param sequences:
+    :return:
+    """
+    chain_id = line[11]
+    if chain_id not in sequences:
+        sequences[chain_id] = []
+    sequence = sequences[chain_id]
+    sequence.extend(line[19:].split())
+
+
+def _pdb_find_missing_residues(mol, sequences, chain, residue_type_map):
+    """
+
+    :param filename:
+    :param sequences:
+    :return:
+    """
+    if sequences:
+        original_residue_names = [GlobalSetting.PDBResidueNameMap["save"].get(name, name) for name in residue_type_map]
+        for chain_id, sequence in sequences.items():
+            pdb_index_to_mol_index = chain[chain_id]
+            name_index_to_mol_index = Xdict()
+            pdb_index = list(pdb_index_to_mol_index.keys())
+            pdb_index.sort()
+            names_with_none = [None] * (pdb_index[-1] - pdb_index[0] + 1)
+            for i in pdb_index:
+                names_with_none[i - pdb_index[0]] = original_residue_names[pdb_index_to_mol_index[i]]
+                name_index_to_mol_index[i - pdb_index[0]] = pdb_index_to_mol_index[i]
+            offset = 0
+            for offset in range(len(sequence) - len(names_with_none) + 1):
+                if all(b in (a, None) for a, b in zip(sequence[offset:], names_with_none)):
+                    break
+            if offset:
+                mol.set_missing_residues_info(None, name_index_to_mol_index[offset], sequence[:offset])
+            tail_offset = len(sequence) - offset - len(names_with_none)
+            if tail_offset:
+                tail_index = name_index_to_mol_index[len(names_with_none) - 1]
+                mol.set_missing_residues_info(tail_index, None, sequence[-tail_offset:])
+            find_none_index = None
+            for i, name in enumerate(names_with_none):
+                if find_none_index is None and name is None:
+                    find_none_index = i
+                elif find_none_index is not None and name is not None:
+                    start = name_index_to_mol_index[find_none_index - 1]
+                    end = name_index_to_mol_index[i]
+                    mol.set_missing_residues_info(start, end, sequence[find_none_index:i])
+                    find_none_index = None
+
+
+def _pdb_connects(mol, connects, atom_map):
+    """
+
+    :param mol:
+    :param connects:
+    :param atom_map:
+    :return:
+    """
+    for connect in connects:
+        atom = int(connect[6:11])
+        if atom not in atom_map:
+            continue
+        atom_ = atom_map[atom]
+        connect = connect[11:]
+        for i in range(0, len(connect), 5):
+            to_connect = connect[i:i+5]
+            if not to_connect.strip():
+                break
+            to_connect = int(to_connect)
+            if to_connect not in atom_map:
+                continue
+            to_connect_ = atom_map[to_connect]
+            if atom > to_connect or atom_.residue == to_connect_.residue or mol.get_residue_link(atom_, to_connect_):
+                continue
+            mol.add_residue_link(atom_, to_connect_)
+            print(atom_, atom, to_connect_, to_connect)
+
+
+def load_pdb(filename, judge_histone=True, position_need="A", ignore_hydrogen=False,
+             ignore_unknown_name=False, ignore_seqres=True, ignore_conect=True):
     """
     This **function** is used to load a pdb file
 
     :param filename: the name of the file to load
     :param judge_histone: judge the protonized state of the histone residues
     :param position_need: the position character to read
-    :param ignore_hydrogen: do not read the atom with a name beginning with "H"
+    :param ignore_hydrogen: do not read the atom with a name beginning with "H" or "[123]H"
     :param ignore_unknown_name: do not read the atom with an unknown name **New From 1.2.6.4**
+    :param ignore_seqres: do not read the SEQRES lines **New From 1.2.6.7**
+    :param ignore_conect: do not read the CONECT lines **New From 1.2.6.7**
     :return: a Molecule instance
     """
     molecule = Molecule(os.path.splitext(os.path.basename(filename))[0])
     chain = Xdict()
+    sequences = Xdict()
     ssbonds = []
+    links = []
+    connects = []
     residue_type_map = []
+    insertion_count = 0
     current_residue_count = -1
+    current_insertion_code = None
     current_residue_index = None
     current_resname = None
     current_histone_information = {"DeltaH": False, "EpsilonH": False}
     with open(filename) as f:
         for line in f:
             if line.startswith("ATOM") or line.startswith("HETATM"):
-                resindex = int(line[22:26])
+                resindex = int(line[22:26]) + insertion_count
+                insertion_code = line[26]
+                chain_id = line[21]
+                if not chain_id.strip():
+                    chain_id = 0
                 resname = line[17:20].strip()
                 atomname = line[12:16].strip()
                 if current_residue_index is None:
                     _pdb_judge_histone(judge_histone, residue_type_map, current_histone_information)
                     current_residue_count += 1
                     current_resname = resname
-                    residue_type_map.append(resname)
-                    current_residue_index = resindex
-                    chain[chr(ord("A") + len(chain.keys()))] = Xdict()
-                    chain[chr(ord("A") + len(chain.keys())-1)][resindex] = current_residue_count
+                    current_insertion_code = insertion_code
                     if resname in GlobalSetting.PDBResidueNameMap["head"].keys():
                         resname = GlobalSetting.PDBResidueNameMap["head"][resname]
-                elif current_residue_index != resindex or current_resname != resname:
+                    residue_type_map.append(resname)
+                    current_residue_index = resindex
+                    chain[chain_id] = Xdict()
+                    chain[chain_id][resindex] = current_residue_count
+                elif (current_residue_index != resindex or current_insertion_code != insertion_code) or \
+                        current_resname != resname:
                     _pdb_judge_histone(judge_histone, residue_type_map, current_histone_information)
                     current_residue_count += 1
                     current_resname = resname
+                    current_insertion_code = insertion_code
+                    if insertion_code != " ":
+                        insertion_count += 1
                     current_residue_index = resindex
-                    chain[chr(ord("A") + len(chain.keys()) - 1)][resindex] = current_residue_count
-
+                    chain[chain_id][resindex] = current_residue_count
                     residue_type_map.append(resname)
-
                 if judge_histone and resname in GlobalSetting.HISMap["HIS"].keys():
                     if atomname == GlobalSetting.HISMap["DeltaH"]:
                         current_histone_information["DeltaH"] = True
                     elif atomname == GlobalSetting.HISMap["EpsilonH"]:
                         current_histone_information["EpsilonH"] = True
+
             elif line.startswith("TER"):
                 current_residue_index = None
                 current_resname = None
+                current_insertion_code = None
+                insertion_count = 0
                 if residue_type_map[-1] in GlobalSetting.PDBResidueNameMap["tail"].keys():
                     residue_type_map[-1] = GlobalSetting.PDBResidueNameMap["tail"][residue_type_map[-1]]
                 _pdb_judge_histone(judge_histone, residue_type_map, current_histone_information)
@@ -293,13 +526,26 @@ def load_pdb(filename, judge_histone=True, position_need="A", ignore_hydrogen=Fa
             elif line.startswith("SSBOND"):
                 ssbonds.append(line)
 
+            elif line.startswith("LINK"):
+                links.append(line)
+
+            elif line.startswith("SEQRES") and not ignore_seqres:
+                _pdb_read_sequences(line, sequences)
+
+            elif line.startswith("CONECT") and not ignore_conect:
+                connects.append(line)
+
     current_residue_index = None
     if residue_type_map[-1] in GlobalSetting.PDBResidueNameMap["tail"].keys():
         residue_type_map[-1] = GlobalSetting.PDBResidueNameMap["tail"][residue_type_map[-1]]
 
     _pdb_ssbond_before(chain, residue_type_map, ssbonds)
-    _pdb_add_residue(filename, molecule, position_need, residue_type_map, ignore_hydrogen, ignore_unknown_name)
+    atom_map = _pdb_add_residue(filename, molecule, position_need,
+                                residue_type_map, ignore_hydrogen, ignore_unknown_name)
     _pdb_ssbond_after(chain, ssbonds, molecule)
+    _pdb_link_after(chain, links, molecule)
+    _pdb_find_missing_residues(molecule, sequences, chain, residue_type_map)
+    _pdb_connects(molecule, connects, atom_map)
 
     return molecule
 
@@ -335,6 +581,15 @@ def load_coordinate(filename, mol=None):
 ##########################################################################
 # amber Format
 ##########################################################################
+def _frcmod_nb14(line, atoms):
+    nb14ee = re.findall(r"SCEE=[\d+\.]+", line)
+    nb14lj = re.findall(r"SCNB=[\d+\.]+", line)
+    if not nb14ee and not nb14lj:
+        return ""
+    nb14ee = 1.0 / float(nb14ee[0].split("=")[1]) if nb14ee else 1.0 / 1.2
+    nb14lj = 1.0 / float(nb14lj[0].split("=")[1]) if nb14lj else 1.0 / 2.0
+    return f"{atoms[0]}-{atoms[3]} {nb14ee} {nb14lj}\n"
+
 def _frcmod_cmap(line, cmap, temp_cmp, cmap_flag):
     """
 
@@ -367,14 +622,16 @@ def _frcmod_cmap(line, cmap, temp_cmp, cmap_flag):
     return temp_cmp, cmap_flag
 
 
-def _frcmod_atoms_words(line, n):
+def _frcmod_atoms_words(line, n, last_atoms=None):
     """
 
     :param line:
     :param n:
     :return:
     """
-    return [word.strip() for word in line[:n].split("-")], line[n:].split()
+    if line[0] != " ":
+        return [word.strip() for word in line[:n].split("-")], line[n:].split()
+    return [last_atoms, line[n:].split()]
 
 
 def load_frcmod(filename, nbtype="RE"):
@@ -466,7 +723,7 @@ def load_parmdat(filename):
     This **function** is used to load a parmdat file
 
     :param filename: the name of the file to load
-    :return: a list of strings, including atoms, bonds, angles, propers, impropers, ljs information respectively
+    :return: a list of strings, including atoms, bonds, angles, propers, impropers, ljs, nb14s information respectively
     """
     with open(filename) as f:
         f.readline()
@@ -492,16 +749,21 @@ def load_parmdat(filename):
         # 读恰当二面角
         reset = 1
         propers = "name  k[kcal/mol]    phi0[degree]    periodicity    reset\n"
+        nb14s = "name    kLJ     kee\n"
+        atoms = None
         for line in f:
             if not line.strip():
                 break
-            atoms, words = _frcmod_atoms_words(line, 11)
+            last_atoms = atoms
+            atoms, words = _frcmod_atoms_words(line, 11, last_atoms)
+            nb14s += _frcmod_nb14(line, atoms)
             propers += "-".join(atoms) + "\t" + str(float(words[1]) / int(words[0])) + "\t" + words[2] + "\t" + str(
                 abs(int(float(words[3])))) + "\t" + str(reset) + "\n"
             if int(float(words[3])) < 0:
                 reset = 0
             else:
                 reset = 1
+
         # 读非恰当二面角
         impropers = "name  k[kcal/mol]    phi0[degree]    periodicity\n"
         for line in f:
@@ -542,7 +804,7 @@ def load_parmdat(filename):
     atoms = "name  mass  LJtype\n"
     for atom, mass in atom_types.items():
         atoms += atom + "\t" + mass + "\t" + lj_types[atom] + "\n"
-    toret = [atoms, bonds, angles, propers, impropers, ljs]
+    toret = [atoms, bonds, angles, propers, impropers, ljs, nb14s]
     return toret
 
 
