@@ -6,7 +6,7 @@ from abc import ABC, abstractmethod
 
 import numpy as np
 from .helper import get_rotate_matrix, ResidueType, Molecule, Residue, set_global_alternative_names, Xdict, \
-    GlobalSetting, set_attribute_alternative_names
+    GlobalSetting, set_attribute_alternative_names, Xopen, Xprint
 from .build import save_sponge_input
 from .load import load_coordinate
 from .forcefield.special.min import save_min_bonded_parameters, do_not_save_min_bonded_parameters
@@ -105,11 +105,11 @@ def impose_dihedral(molecule, atom1, atom2, atom3, atom4, dihedral):
     r23 = crd[molecule.atom_index[atom3]] - crd[molecule.atom_index[atom2]]
     r34 = crd[molecule.atom_index[atom3]] - crd[molecule.atom_index[atom4]]
     r12xr23 = np.cross(r12, r23)
-    r23xr34 = np.cross(r23, r34)
-    cos = np.dot(r12xr23, r23xr34) / np.linalg.norm(r12xr23) / np.linalg.norm(r23xr34)
+    r34xr23 = np.cross(r34, r23)
+    cos = np.dot(r12xr23, r34xr23) / np.linalg.norm(r12xr23) / np.linalg.norm(r34xr23)
     cos = max(-0.999999, min(cos, 0.999999))
     dihedral0 = np.arccos(cos)
-    dihedral0 = np.pi - np.copysign(dihedral0, np.cross(r23xr34, r12xr23).dot(r23))
+    dihedral0 = np.pi - np.copysign(dihedral0, np.cross(r34xr23, r12xr23).dot(r23))
     delta_angle = dihedral - dihedral0
     crd[atom4_friends] = np.dot(crd[atom4_friends] - crd[molecule.atom_index[atom3]],
                                 get_rotate_matrix(r23, delta_angle)) + crd[molecule.atom_index[atom3]]
@@ -371,46 +371,80 @@ def get_peptide_from_sequence(sequence, charged_terminal=True):
     return toret
 
 
-def optimize(mol, step=5000, only_bad_coordinate=True):
+def optimize(mol, step=2000, only_bad_coordinate=True, dt=1e-8, force_limit=50, epoch_limit=10):
     """
     This **function** is used to optimize the structure of the Molecule instance
 
     :param mol: the molecule to optimize
-    :param step: the step to minimize
+    :param step: the limited step for each epoch for minimization, 2000 for default
     :param only_bad_coordinate: whether to optimize all the atoms or the atoms whose coordinates are bad
+    :param dt: the start dt for minimization
+    :param force_limit: the minimization will stop if the largest difference in unit of `kcal/mol/A` \
+between forces in two steps is not more than this value.
+    :param epoch_limit: the minimization will stop if the epoch is not less than this value.
     :return: None
     """
     from tempfile import TemporaryDirectory
+    Xprint("Optimizing", verbose=0)
     with TemporaryDirectory() as tempdir:
         temp_prefix = os.path.join(tempdir, "temp")
         temp_out = os.path.join(tempdir, "min")
+        Xprint("    Parametering", verbose=0)
         save_min_bonded_parameters()
         save_sponge_input(mol, temp_prefix)
         do_not_save_min_bonded_parameters()
-        all_to_use = f"""SPONGE_NOPBC -default_in_file_prefix {temp_prefix} -rst {temp_out} -crd {temp_prefix}.dat
-        -box {temp_prefix}.box -mdout {temp_out}.out -mdinfo {temp_out}.info -mode minimization 
-        -step_limit {step} """
+        temp_mdin_name = os.path.join(tempdir, "mdin.txt")
+        mdin = Xopen(temp_mdin_name, "w")
+        mdin.write(f"""temp
+default_in_file_prefix = {temp_prefix}
+rst = {temp_out}
+crd = {temp_prefix}.dat
+box = {temp_prefix}.box
+mdout = {temp_out}.out
+mdinfo = {temp_out}.info
+mode = minimization
+step_limit = {step}
+write_information_interval = {step}
+""")
+        mdin.close()
+        all_to_use = f"SPONGE -mdin {temp_mdin_name} "
+        if force_limit > 0:
+            all_to_use += f"-frc {temp_out}.frc "
         if only_bad_coordinate:
             all_to_use += f"-mass_in_file {temp_prefix + '_fake_mass.txt'} "
         if GlobalSetting.verbose < 2:
             print_to = f" > {os.devnull}"
         else:
             print_to = ""
-        run(all_to_use + "-dt 1e-8" + print_to)
-
+        Xprint("    Running", verbose=0)
+        run(all_to_use + f"-dt {dt} {print_to}")
         all_to_use += f"-coordinate_in_file {temp_out+'_coordinate.txt'} "
-        for i in [7, 6, 5, 4, 3]:
-            run(all_to_use + f"-dt 1e-{i} {print_to}")
-        load_coordinate(temp_out+"_coordinate.txt", mol)
+        start_dt = dt
+        if force_limit > 0:
+            frc = np.fromfile(f"{temp_out}.frc", dtype=np.float32).reshape(-1, 3)
+            frc = np.linalg.norm(frc, axis=1)
+            last_frc = np.zeros_like(frc)
+            epoch = 0
+            Xprint("Epoch    Max Force Difference    Force Limit    Epoch Limit", verbose=0)
+            while not np.all(np.abs(frc - last_frc) < force_limit) and epoch < epoch_limit:
+                epoch += 1
+                dt = np.max((np.min((1e-3, np.sqrt(1e-4 / np.max(frc)))), start_dt))
+                run(all_to_use + f"-dt {dt} {print_to}")
+                last_frc = frc
+                frc = np.fromfile(f"{temp_out}.frc", dtype=np.float32).reshape(-1, 3)
+                frc = np.linalg.norm(frc, axis=1)
+                Xprint("{0:5d}{1:>24.4f}{2:15.4f}{3:15d}".format(epoch,
+                                                                 np.max(np.abs(frc - last_frc)),
+                                                                 force_limit,
+                                                                 epoch_limit), verbose=0)
+        load_coordinate(temp_out+'_coordinate.txt', mol)
+        Xprint("Optimization Finished", verbose=0)
 
 
 class Region(ABC):
     """
     This **abstract class** is used to define a region
     **New From 1.2.6.4**
-
-    :param *regions: the regions
-    :param do: eigher 'union' or 'intersect'
     """
     def __init__(self):
         self.side = "in"
@@ -609,13 +643,15 @@ class PrismRegion(Region):
 
         return ans if self._side else not ans
 
+
 class Lattice:
     """
     This **class** is used to help with the process of the lattice
     **New From 1.2.6.4**
 
-    :param style: the style of the lattice. "fcc"
-    :param basis_molecule: the unit to repeat, a Residue, a ResidueType or a Molecule.
+    :param style: the style of the lattice. "custom", "template:NAME" or the key values in Lattice.styles
+        "sc", "fcc", "bcc", "hcp", "diamond" are in the default template styles.
+    :param basis_molecule: the unit to repeat, a Residue, a ResidueType or a Molecule
     :param scale: the scale of the lattice
     :param origin: a list with 3 numbers, the origin of the lattice. [0,0,0] for default.
     :param cell_length: the length of the unit cell. [1,1,1] for default.
@@ -703,9 +739,10 @@ class Lattice:
             raise ValueError("Box should only be a BlockRegion with side == 'in' !")
         if not mol:
             mol = Molecule("unnamed")
+        mol.box_length = [box.x_high - box.x_low, box.y_high - box.y_low, box.z_high - box.z_low]
         cell_angle = self.cell_angle * np.pi / 180.0
         c0, c1, c2 = np.cos(cell_angle)
-        s0, s1, s2 = np.sin(cell_angle)
+        _, _, s2 = np.sin(cell_angle)
         basis_mol = self.basis_molecule
         res_len = -1
         if isinstance(basis_mol, Molecule):
@@ -722,7 +759,7 @@ class Lattice:
         dl3z = dl3x * np.sqrt(1 - c2 * c2 - c1 * c1 - c0 * c0 + 2 * c0 * c1 * c2) / s2
         dl3x *= c1
         bps = np.array([[dl1x * basis[0] + dl2x * basis[1] + dl3x * basis[2],
-               dl2y * basis[1] + dl3y * basis[2], dl3z * basis[2]] for basis in self.basis_position])
+                         dl2y * basis[1] + dl3y * basis[2], dl3z * basis[2]] for basis in self.basis_position])
         x_init = 0
         y_init = 0
         z0 = box.z_low + self.origin[2]
@@ -759,9 +796,9 @@ FACE_CENTERED_CUBIC_LATTICE = Lattice("template:fcc", basis_position=[[0, 0, 0],
                                                                       [0, 0.5, 0.5], [0.5, 0.5, 0]])
 
 HEXAGONAL_CLOSE_PACKED_LATTICE = Lattice("template:hcp",
-                                          basis_position=[[0, 0, 0], [1/3, 1/3, 0.5]],
-                                          cell_angle=[90, 90, 60],
-                                          cell_length=[1, 1, 2/3*np.sqrt(6)])
+                                         basis_position=[[0, 0, 0], [1/3, 1/3, 0.5]],
+                                         cell_angle=[90, 90, 60],
+                                         cell_length=[1, 1, 2/3*np.sqrt(6)])
 
 DIAMOND_LATTICE = Lattice("template:diamond", basis_position=[[0, 0, 0], [0, 0.5, 0.5],
                                                               [0.5, 0, 0.5], [0.5, 0.5, 0],
